@@ -31,7 +31,6 @@
 class PINOT_EXPORT FilterWrapper
 {
 	public:
-		/// Builds a FilterWrapper object.
 		FilterWrapper(IndexInterface *pIndex);
 		virtual ~FilterWrapper();
 
@@ -48,10 +47,6 @@ class PINOT_EXPORT FilterWrapper
 	protected:
 		IndexInterface *m_pIndex;
 
-		bool filterDocument(const Document &doc, const std::string &originalType,
-			const std::set<std::string> &labels, unsigned int &docId,
-			bool doUpdate);
-
 		bool unindexNestedDocuments(const std::string &url);
 
 	private:
