@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,13 +22,11 @@
 #include <iostream>
 #include <glibmm/thread.h>
 #include <glibmm/convert.h>
-#include <libxml/parserInternals.h>
-#include <libxml++/parsers/domparser.h>
-#include <libxml++/nodes/node.h>
-#include <libxml++/nodes/textnode.h>
+#include <libxml++/libxml++.h>
 #include <libxml++/nodes/cdatanode.h>
 
 #include "StringManip.h"
+#include "FilterUtils.h"
 #include "OpenSearchParser.h"
 
 using namespace std;
@@ -85,7 +83,7 @@ OpenSearchResponseParser::~OpenSearchResponseParser()
 }
 
 bool OpenSearchResponseParser::parse(const ::Document *pResponseDoc, vector<DocumentInfo> &resultsList,
-	unsigned int &totalResults, unsigned int &firstResultIndex) const
+	unsigned int &totalResults, unsigned int &firstResultIndex, string &charset) const
 {
 	float pseudoScore = 100;
 	unsigned int contentLen = 0;
@@ -112,7 +110,6 @@ bool OpenSearchResponseParser::parse(const ::Document *pResponseDoc, vector<Docu
 	{
 		bool loadFeed = false;
 
-		// Parse the configuration file
 		DomParser parser;
 		parser.set_substitute_entities(true);
 		parser.parse_memory_raw((const unsigned char *)pContent, (Parser::size_type)contentLen);
@@ -122,6 +119,15 @@ bool OpenSearchResponseParser::parse(const ::Document *pResponseDoc, vector<Docu
 			return false;
 		}
 
+		ustring encoding(pDocument->get_encoding());
+		if (encoding.empty() == false)
+		{
+			charset = encoding;
+#ifdef DEBUG
+			cout << "OpenSearchResponseParser::parse: response charset is " << charset << endl;
+#endif
+		}
+
 		Node *pNode = pDocument->get_root_node();
 		Element *pRootElem = dynamic_cast<Element *>(pNode);
 		if (pRootElem == NULL)
@@ -258,6 +264,18 @@ bool OpenSearchResponseParser::parse(const ::Document *pResponseDoc, vector<Docu
 				}
 			}
 
+			// The extract may contain HTML
+			if ((extract.find("<") != string::npos) &&
+				(extract.find(">") != string::npos))
+			{
+				// Wrap the extract
+				ustring dummyHtml("<html><head><meta HTTP-EQUIV=\"content-type\" CONTENT=\"text/html\"></head><body>");
+				dummyHtml += extract;
+				dummyHtml += "</body></html>";
+
+				extract = FilterUtils::stripMarkup(dummyHtml);
+			}
+
 			DocumentInfo result(title, url, "", "");
 			result.setExtract(extract);
 			result.setScore(pseudoScore);
@@ -307,7 +325,6 @@ ResponseParserInterface *OpenSearchParser::parse(SearchPluginProperties &propert
 
 	try
 	{
-		// Parse the configuration file
 		DomParser parser;
 		parser.set_substitute_entities(true);
 		parser.parse_file(m_fileName);
@@ -353,7 +370,8 @@ ResponseParserInterface *OpenSearchParser::parse(SearchPluginProperties &propert
 
 				if (nodeName == "ShortName")
 				{
-					properties.m_name = nodeContent;
+					// Ignore LongName, use this as long name
+					properties.m_longName = nodeContent;
 				}
 				else if (nodeName == "Url")
 				{
@@ -523,10 +541,6 @@ ResponseParserInterface *OpenSearchParser::parse(SearchPluginProperties &propert
 					// This is supposed to be a space-delimited list, but use the whole thing as channel
 					properties.m_channel = nodeContent;
 				}
-				else if (nodeName == "LongName")
-				{
-					properties.m_longName = nodeContent;
-				}
 				else if (nodeName == "Language")
 				{
 					properties.m_languages.insert(nodeContent);
@@ -548,15 +562,23 @@ ResponseParserInterface *OpenSearchParser::parse(SearchPluginProperties &propert
 	}
 
 	// Scrolling
-	properties.m_nextIncrement = 1;
 	properties.m_nextBase = 1;
 	if (properties.m_variableParameters.find(SearchPluginProperties::START_PAGE_PARAM) != properties.m_variableParameters.end())
 	{
 		properties.m_scrolling = SearchPluginProperties::PER_PAGE;
+		properties.m_nextIncrement = 1;
+	}
+	else if ((properties.m_variableParameters.find(SearchPluginProperties::COUNT_PARAM) != properties.m_variableParameters.end()) ||
+		(properties.m_variableParameters.find(SearchPluginProperties::START_INDEX_PARAM) != properties.m_variableParameters.end()))
+	{
+		properties.m_scrolling = SearchPluginProperties::PER_INDEX;
+		properties.m_nextIncrement = 0;
 	}
 	else
 	{
-		properties.m_scrolling = SearchPluginProperties::PER_INDEX;
+		// No scrolling
+		properties.m_nextIncrement = 0;
+		properties.m_nextBase = 0;
 	}
 
 	return new OpenSearchResponseParser(rssResponse);
