@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -220,7 +220,7 @@ SherlockResponseParser::~SherlockResponseParser()
 }
 
 bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<DocumentInfo> &resultsList,
-	unsigned int &totalResults, unsigned int &firstResultIndex) const
+	unsigned int &totalResults, unsigned int &firstResultIndex, string &charset) const
 {
 	float pseudoScore = 100;
 	unsigned int contentLen = 0;
@@ -233,6 +233,22 @@ bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<Document
 		return false;
 	}
 
+	// Can we get the charset ?
+	Dijon::HtmlFilter htmlFilter("text/html");
+	if (FilterUtils::feedFilter(*pResponseDoc, &htmlFilter) == true)
+	{
+		const map<string, string> &metaData = htmlFilter.get_meta_data();
+		map<string, string>::const_iterator charsetIter = metaData.find("charset");
+
+		if (charsetIter != metaData.end())
+		{
+			charset = charsetIter->second;
+#ifdef DEBUG
+			cout << "SherlockResponseParser::parse: response charset is " << charset << endl;
+#endif
+		}
+	}
+
 	// These two are the minimum we need
 	if ((m_resultItemStart.empty() == true) ||
 		(m_resultItemEnd.empty() == true))
@@ -386,26 +402,8 @@ bool SherlockResponseParser::parse(const Document *pResponseDoc, vector<Document
 
 		if (url.empty() == false)
 		{
-			Url urlObj(url);
-
-			// Is this URL relative to the search engine's domain ?
 			// FIXME: look for a interpret/baseurl tag, see https://bugzilla.mozilla.org/show_bug.cgi?id=65453
 			// FIXME: obey m_skipLocal
-			if (urlObj.getHost().empty() == true)
-			{
-				Url baseUrlObj(pResponseDoc->getLocation());
-
-				string tmpUrl = baseUrlObj.getProtocol();
-				tmpUrl += "://";
-				tmpUrl += baseUrlObj.getHost();
-				if (url[0] != '/')
-				{
-					tmpUrl += "/";
-				}
-				tmpUrl += url;
-				url = tmpUrl;
-			}
-
 			DocumentInfo result(name, url, "", "");
 			result.setExtract(extract);
 			result.setScore(pseudoScore);
@@ -524,7 +522,7 @@ ResponseParserInterface *SherlockParser::parse(SearchPluginProperties &propertie
 		map<string, string>::iterator mapIter = lowSearchParams.find("name");
 		if (mapIter != lowSearchParams.end())
 		{
-			properties.m_name = mapIter->second;
+			properties.m_longName = mapIter->second;
 		}
 
 		// Channel
