@@ -1,5 +1,5 @@
 /*
- *  Copyright 2008 Fabrice Colin
+ *  Copyright 2008-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 
 #include <unistd.h>
 #include <stdarg.h>
+#include <stdlib.h>
+#include <string.h>
 #include <time.h>
 #include <iostream>
 #include <libexif/exif-data.h>
@@ -25,6 +27,7 @@
 #include <libexif/exif-loader.h>
 #include <libexif/exif-utils.h>
 
+#include "config.h"
 #include "ExifImageFilter.h"
 
 using std::string;
@@ -60,11 +63,17 @@ DIJON_FILTER_EXPORT Filter *get_filter(const std::string &mime_type)
 }
 #endif
 
-struct ExifMetaData
+class ExifMetaData
 {
-	string m_title;
-	string m_date;
-	string m_content;
+	public:
+		ExifMetaData(dstring &content) :
+			m_content(content)
+		{
+		}
+
+		string m_title;
+		string m_date;
+		dstring &m_content;
 };
 
 static void entryCallback(ExifEntry *pEntry, void *pData)
@@ -90,11 +99,30 @@ static void entryCallback(ExifEntry *pEntry, void *pData)
 			pMetaData->m_title = value;
 			break;
 		case EXIF_TAG_DATE_TIME:
+#ifdef HAVE_STRPTIME
 			if (strptime(value, "%Y:%m:%d %H:%M:%S", &timeTm) != NULL)
+#else
+			{
+				string valueStr(value);
+
+				timeTm.tm_year = atoi(valueStr.substr(0, 4).c_str());
+				timeTm.tm_mon = atoi(valueStr.substr(5, 2).c_str());
+				timeTm.tm_mday = atoi(valueStr.substr(8, 2).c_str());
+				timeTm.tm_hour = atoi(valueStr.substr(11, 2).c_str());
+				timeTm.tm_min = atoi(valueStr.substr(14, 2).c_str());
+				timeTm.tm_sec = atoi(valueStr.substr(17, 2).c_str());
+			}
+			if (timeTm.tm_mday > 0)
+#endif
 			{
 				char timeStr[64];
 
+#if defined(__GNU_LIBRARY__)
+				// %z is a GNU extension
 				if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %z", &timeTm) > 0)
+#else
+				if (strftime(timeStr, 64, "%a, %d %b %Y %H:%M:%S %Z", &timeTm) > 0)
+#endif
 				{
 					pMetaData->m_date = timeStr;
 				}
@@ -102,7 +130,7 @@ static void entryCallback(ExifEntry *pEntry, void *pData)
 			break;
 		default:
 			pMetaData->m_content += " ";
-			pMetaData->m_content += value;
+			pMetaData->m_content.append(value, strlen(value));
 			break;
 	}
 #ifdef DEBUG
@@ -192,7 +220,7 @@ bool ExifImageFilter::next_document(void)
 		}
 		else
 		{
-			ExifMetaData *pMetaData = new ExifMetaData();
+			ExifMetaData *pMetaData = new ExifMetaData(m_content);
 
 			// Get it all
 			exif_data_foreach_content(pData, contentCallback, pMetaData);
@@ -202,7 +230,6 @@ bool ExifImageFilter::next_document(void)
 			{
 				m_metaData["date"] = pMetaData->m_date;
 			}
-			m_metaData["content"] = pMetaData->m_content;
 
 			delete pMetaData;
 			exif_data_unref(pData);
