@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2008 Fabrice Colin
+ *  Copyright 2007-2009 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <fcntl.h>
 #ifdef HAVE_SOCKETPAIR
 #ifdef HAVE_FORK
 #ifdef HAVE_SETRLIMIT
@@ -36,15 +37,19 @@
 #endif
 #endif
 #include <unistd.h>
+#include <string.h>
 #include <sstream>
+#include <algorithm>
 #include <iostream>
-#include <cstring>
 #include <libxml/xmlreader.h>
 
 #include "ExternalFilter.h"
 
+#define MAX_OUTPUT_SIZE 5242880
+
 using std::cout;
 using std::endl;
+using std::min;
 using std::string;
 using std::stringstream;
 using std::set;
@@ -114,6 +119,55 @@ static string shell_protect(const string &file_name)
 	return safefile;
 }
 
+static bool read_file(int fd, bool limit_output, ssize_t &totalSize, dstring &fileBuffer)
+{
+	struct stat fdStats;
+	ssize_t bytesRead = 0;
+	bool gotOutput = true;
+
+	if (fstat(fd, &fdStats) == 0)
+	{
+		if (fdStats.st_size > 0)
+		{
+			fileBuffer.reserve(fdStats.st_size);
+		}
+	}
+
+	do
+	{
+		if ((limit_output == true) && 
+			(totalSize >= MAX_OUTPUT_SIZE))
+		{
+#ifdef DEBUG
+			cout << "ExternalFilter::read_file: stopping at " << totalSize << endl;
+#endif
+			break;
+		}
+
+		char readBuffer[4096];
+		bytesRead = read(fd, readBuffer, 4096);
+		if (bytesRead > 0)
+		{
+			fileBuffer.append(readBuffer, bytesRead);
+			totalSize += bytesRead;
+		}
+		else if (bytesRead == -1)
+		{
+			// An error occured
+			if (errno != EINTR)
+			{
+				gotOutput = false;
+				break;
+			}
+
+			// Try again
+			bytesRead = 1;
+		}
+	} while (bytesRead > 0);
+
+	return gotOutput;
+}
+
 map<string, string> ExternalFilter::m_commandsByType;
 map<string, string> ExternalFilter::m_outputsByType;
 map<string, string> ExternalFilter::m_charsetsByType;
@@ -177,9 +231,12 @@ bool ExternalFilter::next_document(void)
 		(m_filePath.empty() == false) &&
 		(m_commandsByType.empty() == false))
 	{
+		string outputType("text/plain");
+		bool limitOutput = true;
+
 		m_doneWithDocument = true;
 
-		// Is this type supported ?
+		// Is this type supported ? Assume text/plain if not specified
 		map<string, string>::const_iterator commandIter = m_commandsByType.find(m_mimeType);
 		if ((commandIter == m_commandsByType.end()) ||
 			(commandIter->second.empty() == true))
@@ -187,21 +244,23 @@ bool ExternalFilter::next_document(void)
 			return false;
 		}
 
-		if (run_command(commandIter->second) == true)
+		// What's the output type ?
+		map<string, string>::const_iterator outputIter = m_outputsByType.find(m_mimeType);
+		if (outputIter != m_outputsByType.end())
+		{
+			outputType = outputIter->second;
+		}
+
+		if (outputType != "text/plain")
+		{
+			limitOutput = false;
+		}
+
+		if (run_command(commandIter->second, limitOutput) == true)
 		{
 			// Fill in general details
 			m_metaData["uri"] = "file://" + m_filePath;
-			// What's the output type ?
-			map<string, string>::const_iterator outputIter = m_outputsByType.find(m_mimeType);
-			if (outputIter == m_outputsByType.end())
-			{
-				// Assume it's plain text if undefined
-				m_metaData["mimetype"] = "text/plain";
-			}
-			else
-			{
-				m_metaData["mimetype"] = outputIter->second;
-			}
+			m_metaData["mimetype"] = outputType;
 			// Is it in a known charset ?
 			map<string, string>::const_iterator charsetIter = m_charsetsByType.find(m_mimeType);
 			if (charsetIter != m_charsetsByType.end())
@@ -242,14 +301,11 @@ void ExternalFilter::initialize(const std::string &config_file, set<std::string>
 
 	types.clear();
 
-	// Initialize the library and check for potential ABI mismatches
-	LIBXML_TEST_VERSION
-
 	// Parse the file and get the document
 #if LIBXML_VERSION < 20600
 	pDoc = xmlParseFile(config_file.c_str());
 #else
-	pDoc = xmlReadFile(config_file.c_str(), NULL, 0);
+	pDoc = xmlReadFile(config_file.c_str(), NULL, XML_PARSE_NOCDATA);
 #endif
 	if (pDoc == NULL)
 	{
@@ -341,9 +397,6 @@ void ExternalFilter::initialize(const std::string &config_file, set<std::string>
 
 	// Free the document
 	xmlFreeDoc(pDoc);
-
-	// Cleanup
-	xmlCleanupParser();
 }
 
 void ExternalFilter::rewind(void)
@@ -353,28 +406,43 @@ void ExternalFilter::rewind(void)
 }
 
 // This function is heavily inspired by Xapian Omega's stdout_to_string()
-bool ExternalFilter::run_command(const string &command)
+bool ExternalFilter::run_command(const string &command, bool limit_output)
 {
-	bool gotOutput = false;
 	string commandLine(command);
-	string::size_type argPos = commandLine.find("%s");
+	bool replacedParam = false, gotOutput = false;
 
-	if (argPos == string::npos)
-	{
-		commandLine += " ";
-		commandLine += shell_protect(m_filePath);
-	}
-	else
+	string::size_type argPos = commandLine.find("%s");
+	while (argPos != string::npos)
 	{
 		string quotedFilePath(shell_protect(m_filePath));
 
 		commandLine.replace(argPos, 2, quotedFilePath);
+		replacedParam = true;
+
+		// Next
+		argPos = commandLine.find("%s", argPos + 1);
+	}
+
+	if (replacedParam == false)
+	{
+		// Append
+		commandLine += " ";
+		commandLine += shell_protect(m_filePath);
 	}
 
 #ifndef LIMIT_EXTERNAL_PROGRAMS
 	// Create a temporary file for the program's output
 	char outTemplate[18] = "/tmp/filterXXXXXX";
+#ifdef HAVE_MKSTEMP
 	int outFd = mkstemp(outTemplate);
+#else
+	int outFd = -1;
+	char *pOutFile = mktemp(outTemplate);
+	if (pOutFile != NULL)
+	{
+		outFd = open(pOutFile, O_WRONLY|O_TRUNC);
+	}
+#endif
 	if (outFd == -1)
 	{
 		return false;
@@ -395,47 +463,8 @@ bool ExternalFilter::run_command(const string &command)
 		return false;
 	}
 
-	struct stat outStats;
-	if (fstat(outFd, &outStats) == 0)
-	{
-		if (outStats.st_size == 0)
-		{
-			// Obviously something went haywire but be graceful
-			m_metaData["content"] = "";
-			m_metaData["size"] = "0";
-			gotOutput = true;
-		}
-		else
-		{
-			// Grab the output
-			char *fileBuffer = new char[outStats.st_size + 1];
-			if (fileBuffer != NULL)
-			{
-				ssize_t bytesRead = read(outFd, (void*)fileBuffer, outStats.st_size);
-				if (bytesRead > 0)
-				{
-					fileBuffer[bytesRead] = '\0';
-
-					m_metaData["content"] = string(fileBuffer, (unsigned int)bytesRead);
-					stringstream numStream;
-					numStream << outStats.st_size;
-					m_metaData["size"] = numStream.str();
-					gotOutput = true;
-				}
-#ifdef DEBUG
-				else cout << "ExternalFilter::run_command: couldn't read output" << endl;
-#endif
-
-				delete[] fileBuffer;
-			}
-#ifdef DEBUG
-			else cout << "ExternalFilter::run_command: couldn't allocate output" << endl;
-#endif
-		}
-	}
-#ifdef DEBUG
-	else cout << "ExternalFilter::run_command: couldn't stat output" << endl;
-#endif
+	ssize_t totalSize = 0;
+	gotOutput = read_file(outFd, limit_output, totalSize, m_content);
 
 	// Close and delete the temporary file
 	close(outFd);
@@ -446,8 +475,6 @@ bool ExternalFilter::run_command(const string &command)
 		return false;
 	}
 #else
-	string fileBuffer;
-	ssize_t bytesRead = 0, totalSize = 0;
 	int status = 0;
 
 	// We want to be able to get the exit status of the child process
@@ -493,30 +520,8 @@ bool ExternalFilter::run_command(const string &command)
 		return false;
 	}
 
-	gotOutput = true;
-	do
-	{
-		char readBuffer[4096];
-
-		bytesRead = read(fds[0], readBuffer, 4096);
-		if (bytesRead > 0)
-		{
-			fileBuffer.append(readBuffer, bytesRead);
-			totalSize += bytesRead;
-		}
-		else if (bytesRead == -1)
-		{
-			// An error occured
-			if (errno != EINTR)
-			{
-				gotOutput = false;
-				break;
-			}
-
-			// Try again
-			bytesRead = 1;
-		}
-	} while (bytesRead > 0);
+	ssize_t totalSize = 0;
+	gotOutput = read_file(fds[0], limit_output, totalSize, m_content);
 
 	// Close our side of the socket pair
 	close(fds[0]);
@@ -549,7 +554,6 @@ bool ExternalFilter::run_command(const string &command)
 	}
 #endif
 
-	m_metaData["content"] = fileBuffer;
 	stringstream numStream;
 	numStream << totalSize;
 	m_metaData["size"] = numStream.str();
