@@ -16,6 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -23,7 +24,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <strings.h>
+#ifdef HAVE_DLFCN_H
 #include <dlfcn.h>
+#endif
 #include <algorithm>
 #include <iostream>
 
@@ -31,11 +34,13 @@
 #include "TextFilter.h"
 #include "FilterFactory.h"
 
+#ifdef HAVE_DLFCN_H
 #ifdef __CYGWIN__
 #define DLOPEN_FLAGS RTLD_LAZY
 #else
 #define DLOPEN_FLAGS (RTLD_LAZY|RTLD_LOCAL)
 #endif
+#endif
 
 #define GETFILTERTYPESFUNC	"_Z16get_filter_typesRSt3setISsSt4lessISsESaISsEE"
 #define GETFILTERFUNC		"_Z10get_filterRKSs"
@@ -62,8 +67,9 @@ FilterFactory::~FilterFactory()
 
 unsigned int FilterFactory::loadFilters(const string &dir_name)
 {
-	struct stat fileStat;
 	unsigned int count = 0;
+#ifdef HAVE_DLFCN_H
+	struct stat fileStat;
 
 	if (dir_name.empty() == true)
 	{
@@ -153,6 +159,7 @@ unsigned int FilterFactory::loadFilters(const string &dir_name)
 		pDirEntry = readdir(pDir);
 	}
 	closedir(pDir);
+#endif
 
 	return count;
 }
@@ -187,6 +194,7 @@ Filter *FilterFactory::getLibraryFilter(const string &mime_type)
 		return NULL;
 	}
 
+#ifdef HAVE_DLFCN_H
 	// Get a filter object then
 	get_filter_func *pFunc = (get_filter_func *)dlsym(pHandle,
 		GETFILTERFUNC);
@@ -196,6 +204,7 @@ Filter *FilterFactory::getLibraryFilter(const string &mime_type)
 	}
 #ifdef DEBUG
 	cout << "FilterFactory::getLibraryFilter: couldn't find export getFilter" << endl;
+#endif
 #endif
 
 	return NULL;
@@ -287,6 +296,7 @@ bool FilterFactory::isSupportedType(const string &mime_type)
 
 void FilterFactory::unloadFilters(void)
 {
+#ifdef HAVE_DLFCN_H
 	for (map<string, void*>::iterator iter = m_handles.begin(); iter != m_handles.end(); ++iter)
 	{
 		if (dlclose(iter->second) != 0)
@@ -296,6 +306,7 @@ void FilterFactory::unloadFilters(void)
 #endif
 		}
 	}
+#endif
 
 	m_types.clear();
 	m_handles.clear();
