@@ -1,5 +1,5 @@
 /*
- *  Copyright 2005,2006 Fabrice Colin
+ *  Copyright 2005-2008 Fabrice Colin
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -9,7 +9,7 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
@@ -21,13 +21,18 @@
 
 #include <string>
 
+#include "Visibility.h"
+
 /// Utility class for string manipulation.
-class StringManip
+class PINOT_EXPORT StringManip
 {
 	public:
 		/// Converts to lowercase.
 		static std::string toLowerCase(const std::string &str);
 
+		/// Strips diacritics.
+		static std::string stripDiacritics(const std::string &str);
+
 		/// Extracts the sub-string between start and end.
 		static std::string extractField(const std::string &str, const std::string &start,
 			const std::string &end, bool anyCharacterOfEnd = false);
